/*
 * Created on 14.07.2005 for PIROL
 *
 * SVN header information:
 *  $Author: LBST-PF-3\orahn $
 *  $Rev: 2509 $
 *  $Date: 2006-10-06 12:01:50 +0200 (Fr, 06 Okt 2006) $
 *  $Id: ValueToColorTableModel.java 2509 2006-10-06 10:01:50Z LBST-PF-3\orahn $
 */
package de.fhOsnabrueck.jump.pirol.ui.tables;

import java.awt.Color;

import de.fhOsnabrueck.jump.pirol.ui.eventHandling.ValueToColorMapChangeListener;
import de.fhOsnabrueck.jump.pirol.utilities.colors.ValueToColorMap;
import de.fhOsnabrueck.jump.pirol.utilities.colors.Value.ColoredRange;
import de.fhOsnabrueck.jump.pirol.utilities.colors.Value.ColoredValue;
import de.fhOsnabrueck.jump.pirol.utilities.comparisonAndSorting.ObjectComparator;
import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.DebugUserIds;
import de.fhOsnabrueck.jump.pirol.utilities.debugOutput.PersonalLogger;
import de.fhOsnabrueck.jump.pirol.utilities.i18n.PirolPlugInMessages;

/**
 * Table model for use with a ValueToColorMap. 
 *
 * @author Ole Rahn
 * <br>
 * <br>FH Osnabr&uuml;ck - University of Applied Sciences Osnabr&uuml;ck,
 * <br>Project: PIROL (2005),
 * <br>Subproject: Daten- und Wissensmanagement
 * 
 * @version $Rev: 2509 $
 * 
 */
public class ValueToColorTableModel extends StandardPirolTableModel implements ValueToColorMapChangeListener {
    
    private static final long serialVersionUID = 6944868368034171963L;
    
    protected Class<Object>[] colClasses = null;
    
    public int[] colWidths = {25,50,50,15,15};
    public boolean[] editables = { true, true, true, true, true };
    
    protected ValueToColorMap val2color = null;
    
    protected PersonalLogger logger = new PersonalLogger(DebugUserIds.OLE);
    
    protected boolean enabled = true;
    
    protected final static String[] colNamesNum = new String[]{PirolPlugInMessages.getString("color"), PirolPlugInMessages.getString("from"), PirolPlugInMessages.getString("to"), PirolPlugInMessages.getString("lower-border-included"), PirolPlugInMessages.getString("upper-border-included")};
    protected final static Class[]  colClassesNum = new Class[]{ Color.class, Double.class, Double.class, Boolean.class, Boolean.class };
    protected static int[] colWidthsNum = {25,50,50,15,15};
    protected final static boolean[] editablesNum = { true, true, true, true, true };
    
    protected final static String[] colNamesStr = new String[]{PirolPlugInMessages.getString("color"), PirolPlugInMessages.getString("value")};
    protected final static Class[]  colClassesStr = new Class[]{ Color.class, String.class };
    protected static int[] colWidthsStr = {25,120};
    protected final static boolean[] editablesStr = { true, false };

    public ValueToColorTableModel(ValueToColorMap val2color){
        super(colNamesNum);

        this.setVal2color(val2color);
    }
    
    
    
    /**
     *@inheritDoc
     */
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        return (editables[columnIndex] && this.enabled) || (editables[columnIndex] && columnIndex==0);
    }
    
    

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        super.setValueAt(aValue, rowIndex, columnIndex);
        ColoredValue[] cvs = this.val2color.getKeysArray();
        
        switch (columnIndex){
            case (0):
                this.val2color.setColor(rowIndex, (Color)aValue);
                break;
            case (1):
                ((ColoredRange)cvs[rowIndex]).setMinValue(ObjectComparator.getDoubleValue(this.getValueAt(rowIndex, columnIndex)));
                break;
            case (2):
                ((ColoredRange)cvs[rowIndex]).setMaxValue(ObjectComparator.getDoubleValue(this.getValueAt(rowIndex, columnIndex)));
                break;
            case (3):
                ((ColoredRange)cvs[rowIndex]).setIncludingMin( ((Boolean)(this.getValueAt(rowIndex, columnIndex))).booleanValue() );
                break;
            case (4):
                ((ColoredRange)cvs[rowIndex]).setIncludingMax( ((Boolean)(this.getValueAt(rowIndex, columnIndex))).booleanValue() );
                break;
            default:
                break;                
        }
    }

    /**
     *@inheritDoc
     */
    public Class<Object> getColumnClass(int columnIndex) {
        if (this.colClasses.length > columnIndex)
            return this.colClasses[columnIndex];
        return null;
    }
    
    public void rebuildTable(){
        this.clearTable();
        
        ColoredValue[] cvs = this.val2color.getKeysArray();
        
        if (cvs==null){
            this.logger.printError("No val2Color keys!");
            return;
        }

        if (this.val2color.isNumericValues()){
            for (int i=0; i<cvs.length; i++){
                this.addRow(new Object[]{this.val2color.getColorFor(cvs[i]), new Double(((ColoredRange)cvs[i]).getMinValue()), new Double(((ColoredRange)cvs[i]).getMaxValue()), new Boolean(((ColoredRange)cvs[i]).isIncludingMin()), new Boolean(((ColoredRange)cvs[i]).isIncludingMax())});            
            }
        } else {
            for (int i=0; i<cvs.length; i++){
                this.addRow(new Object[]{this.val2color.getColorFor(cvs[i]), cvs[i].toString()});            
            }
        }
    }

    public ValueToColorMap getVal2color() {
        return val2color;
    }

    public void setVal2color(ValueToColorMap val2color) {
        this.val2color = val2color;
        this.clearTable();
        if (val2color.isNumericValues()){
            colClasses = colClassesNum;
            colNames = colNamesNum;
            colWidths = colWidthsNum;
            editables = editablesNum;
            this.setEnabled(true);
        } else {
            colClasses = colClassesStr;
            colNames = colNamesStr;
            colWidths = colWidthsStr;
            editables = editablesStr;
            this.setEnabled(false);
        }
        this.val2color.addValueToColorMapChangeListener(this);
        this.rebuildTable();
    }

    public boolean isEnabled() {
        return enabled;
    }

    public void setEnabled(boolean enabled) {
        this.enabled = enabled;
    }

    /**
     *@param source
     */
    public void valueToColorMapChanged(ValueToColorMap source) {
        this.rebuildTable();
    }
}
